//*****************************************************************************
//                               PnlAnaBase.cpp                               *
//                              ----------------                              *
//  Started     : 26/04/2004                                                  *
//  Last Update : 26/06/2007                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "base/PnlAnaBase.hpp"

//*****************************************************************************
// Constructor.
//
// Argument List:
//   poWin - A pointer to the parent window

PnlAnaBase::PnlAnaBase( wxWindow * poWin ) : wxPanel( poWin )
{
  m_eSimrType = Simulation::eSIMR_NONE;
  m_eAnaType  = Simulation::eANA_NONE;

  CreateBase( );  // Create the analysis panel
  bClear( );      // Clear all object attributes
}

//*****************************************************************************
// Destructor.

PnlAnaBase::~PnlAnaBase( )
{
}

//*****************************************************************************
// Clear the object attributes.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PnlAnaBase::bClear( void )
{
  bool  bRtn=TRUE;

  // Set sweep default values
  if( ! m_oPnlStart.bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlStop .bClear( ) ) bRtn = FALSE;
  if( ! m_oPnlStep .bClear( ) ) bRtn = FALSE;

  // Set input source default values
  if( bIsCreatedSrc( ) && m_oSbxSrc.IsShown( ) )
  {
    m_oChoSrcCpnt.Clear( );
    m_oChoSrcCpnt.Append( wxT("None") );
    m_oChoSrcCpnt.SetSelection( 0 );
    if( m_oPnlSrcLvl.GetParent( ) != NULL )
    {
      m_oPnlSrcLvl.bSetValue( (float) 0.0 );
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_NONE );
    }
  }

  // Set parameters check box default values
  m_oCbxVoltage.SetValue( TRUE );
  m_oCbxCurrent.SetValue( FALSE );
  m_oCbxPower  .SetValue( FALSE );
  m_oCbxResist .SetValue( FALSE );

  // Set the complex part defaults values
  if( bIsCreatedCpxPrt( ) && m_oSbxCpxPrt.IsShown( ) )
  {
    m_oCbxMag  .SetValue( TRUE );
    m_oCbxPhase.SetValue( FALSE );
    m_oCbxReal .SetValue( FALSE );
    m_oCbxImag .SetValue( FALSE );
    m_oCbxMagDb.SetValue( TRUE );
  }

  // Set the analysis temperature default value
  if( bIsCreatedTemp( ) && m_oSbxTemp.IsShown( ) )
    if( ! m_oPnlTemp.bClear( ) )
      bRtn = FALSE;

  m_osErrMsg.Empty( ); // Clear the error string

  return( bRtn );
}

//*****************************************************************************
// Create the basic display objects.

void  PnlAnaBase::CreateBase( void )
{
  if( bIsCreatedBase( ) ) return;

  // Create and add sweep parameter labels
  m_oSbxSwpPars.Create( this, ID_UNUSED, wxT(" Sweep Parameters "),
                        wxPoint( 6, 5 ), wxSize( 330, 120 ) );
  m_oPnlStart.bCreate( this, ID_PNL_START, 120, wxPoint( 18, 28 ) );
  m_oPnlStop .bCreate( this, ID_PNL_STOP,  120, wxPoint( 18, 58 ) );
  m_oPnlStep .bCreate( this, ID_PNL_STEP,  120, wxPoint( 18, 88 ) );
  m_oPnlStart.bSetName( wxT("Start Value") );
  m_oPnlStop .bSetName( wxT("Stop Value") );
  m_oPnlStep .bSetName( wxT("Step Increment") );

  // Create and add simulation parameter check boxes
  m_oSbxCalcPars.Create( this, ID_UNUSED, wxT(" Parameters "),
                         wxPoint( 343, 5 ), wxSize( 105, 122 ) );
  m_oCbxVoltage .Create( this, ID_CBX_VOLT, wxT("Voltage"),
                         wxPoint( 349, 25 ) );
  m_oCbxCurrent .Create( this, ID_CBX_CURR, wxT("Current"),
                         wxPoint( 349, 50 ) );
  m_oCbxPower   .Create( this, ID_CBX_PWR,  wxT("Power"),
                         wxPoint( 349, 75 ) );
  m_oCbxResist  .Create( this, ID_CBX_RES,  wxT("Resistance"),
                         wxPoint( 349, 100 ) );

  // Create and add .OPTIONS configuration dialog button
  m_oBtnOPTIONS.Create( this, ID_BTN_OPTIONS, wxT(".OPTIONS ..."),
                        wxPoint( 343, 137 ), wxSize( 105, 29 ) );
}

//*****************************************************************************
// Create the source component display objects.

void  PnlAnaBase::CreateSrc( void )
{
  if( bIsCreatedSrc( ) ) return;

  // Create and add input source
  m_oSbxSrc    .Create ( this, ID_UNUSED, wxT(" Source "),
                         wxPoint(  6, 175 ), wxSize( 330, 55 ) );
  m_oChoSrcCpnt.Create ( this, ID_CHO_SRCCPNT,
                         wxPoint( 18, 195 ), wxSize( 121, -1 ) );
  m_oPnlSrcLvl .bCreate( this, ID_PNL_SRCLVL, -1, wxPoint( 143, 193 ) );
}

//*****************************************************************************
// Create the simulation parameter complex part display objects.

void  PnlAnaBase::CreateCpxPrt( void )
{
  if( bIsCreatedCpxPrt( ) ) return;

  // Create and add simulation parameter complex part check boxes
  m_oSbxCpxPrt.Create( this, ID_UNUSED,    wxT(" Complex Parts "),
                       wxPoint( 455,  5 ), wxSize( 110, 146 ) );
  m_oCbxMag   .Create( this, ID_CBX_MAG,   wxT("Magnitude"),
                       wxPoint( 461, 24 ) );
  m_oCbxPhase .Create( this, ID_CBX_PHASE, wxT("Phase"),
                       wxPoint( 461, 49 ) );
  m_oCbxReal  .Create( this, ID_CBX_REAL,  wxT("Real Part"),
                       wxPoint( 461, 74 ) );
  m_oCbxImag  .Create( this, ID_CBX_IMAG,  wxT("Imag. Part"),
                       wxPoint( 461, 99 ) );
  m_oCbxMagDb .Create( this, ID_CBX_MAGDB, wxT("Mag. in dBV"),
                       wxPoint( 461,124 ) );
}

//*****************************************************************************
// Create the analysis temperature display objects.

void  PnlAnaBase::CreateTemp( void )
{
  if( bIsCreatedTemp( ) ) return;

  // Create and add analysis temperature
  m_oSbxTemp. Create( this, ID_UNUSED, wxT(" Temperature "),
                      wxPoint( 343, 175 ), wxSize( 205, 55 ) );
  m_oPnlTemp.bCreate( this, ID_PNL_TEMP, -1,
                      wxPoint( 355, 193 ) );
  m_oPnlTemp.bSetUnitsType( ChoUnits::eUNITS_TEMP );
}

//*****************************************************************************
// Set the source component values.
//
// Argument List:
//   roSim - The Simulation object

void  PnlAnaBase::SetSrcCpnt( Simulation & roSim )
{
  wxString  os1;

  if( ! bIsCreatedSrc( ) ) return;

  // Set the source component values
  if( ! roSim.rosGetSrcCpnt( ).IsEmpty( ) )
  { // Extract and set the source component label
    os1 = roSim.rosGetSrcCpnt( ).BeforeFirst( wxT(' ') );
    m_oChoSrcCpnt.SetStringSelection( os1 );
    if( m_oPnlSrcLvl.GetParent( ) != NULL )
    { // Extract and set the source level
      SetSrcUType( );
      os1 = roSim.rosGetSrcCpnt( ).AfterLast( wxT(' ') );
      m_oPnlSrcLvl.bSetValue( os1 );
    }
  }
  else
  { // Set the default source component label
    m_oChoSrcCpnt.SetStringSelection( wxT("None") );    
    if( m_oPnlSrcLvl.GetParent( ) != NULL )
    { // Set the default source level
      m_oPnlSrcLvl.bSetValue( (float) 0.0 );
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_NONE );
    }
  }
}

//*****************************************************************************
// Set the source units type based on the type of the source component.

void  PnlAnaBase::SetSrcUType( void )
{
  if( m_oPnlSrcLvl.GetParent( ) == NULL ) return;

  switch( wxToupper( m_oChoSrcCpnt.GetStringSelection( ).GetChar( 0 ) ) )
  {
    case wxT('C'): // Units of capacitance
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_CAP );
      break;

    case wxT('L'): // Units of inductance
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_IND );
      break;

    case wxT('R'): // Units of resistance
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_RES );
      break;

    case wxT('I'): // Units of current
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_CURR );
      break;

    case wxT('V'): // Units of voltage
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_VOLT );
      break;

    default: // No units
      m_oPnlSrcLvl.bSetUnitsType( ChoUnits::eUNITS_NONE );
      break;
  }
}

//*****************************************************************************
// Select the simulator engine to be used.
//
// Arguments:
//   eSimrType - The simulator engine type
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PnlAnaBase::bSetSimrType( Simulation::eSimrType eSimrType )
{
  if( eSimrType<Simulation::eSIMR_FST || eSimrType>Simulation::eSIMR_LST )
    return( FALSE );

  m_eSimrType = eSimrType;

  return( TRUE );
}

//*****************************************************************************
// Select the analysis to be performed.
//
// Arguments:
//   eAnaType - The analysis type
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PnlAnaBase::bSetAnaType( Simulation::eAnaType eAnaType )
{
  if( eAnaType<Simulation::eANA_FST || eAnaType>Simulation::eANA_LST )
    return( FALSE );

  m_eAnaType = eAnaType;

  return( TRUE );
}

//*****************************************************************************
// Load information from a Simulation object.
//
// Argument List:
//   roSim - The Simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PnlAnaBase::bLoad( Simulation & roSim )
{
  bool  bRtn=TRUE;

  // Set the sweep values
  if( ! m_oPnlStart.bSetValue( (double) roSim.fGetSwpStart( ) ) ) bRtn = FALSE;
  if( ! m_oPnlStop .bSetValue( (double) roSim.fGetSwpStop( )  ) ) bRtn = FALSE;
  if( ! m_oPnlStep .bSetValue( (double) roSim.fGetSwpStep( )  ) ) bRtn = FALSE;
  if( m_oPnlStart.dfGetValue( ) == 0.0 )
    m_oPnlStart.bSetUnits( m_oPnlStop.rosGetUnits( ) );

  // Set the source component
  SetSrcCpnt( roSim );

  // Set the parameters to derive
  m_oCbxVoltage.SetValue( roSim.bGetOutPar( Simulation::ePAR_VLT ) );
  if( m_eSimrType != Simulation::eSIMR_NGSPICE )
  {
    m_oCbxCurrent.SetValue( roSim.bGetOutPar( Simulation::ePAR_CUR ) );
    m_oCbxPower  .SetValue( roSim.bGetOutPar( Simulation::ePAR_PWR ) );
    m_oCbxResist .SetValue( roSim.bGetOutPar( Simulation::ePAR_RES ) );
  }

  // Set the complex parts to derive if this is an AC analysis
  if( m_eAnaType == Simulation::eANA_AC )
  {
    m_oCbxMag  .SetValue( roSim.bGetOutCpx( Simulation::eCPX_MAG   ) );
    m_oCbxPhase.SetValue( roSim.bGetOutCpx( Simulation::eCPX_PHASE ) );
    m_oCbxReal .SetValue( roSim.bGetOutCpx( Simulation::eCPX_REAL  ) );
    m_oCbxImag .SetValue( roSim.bGetOutCpx( Simulation::eCPX_IMAG  ) );
    m_oCbxMagDb.SetValue( roSim.bGetOutCpx( Simulation::eCPX_MAGDB ) );
  }

  // Set the analysis temperature
  if( bIsCreatedTemp( ) )
    if( ! m_oPnlTemp.bSetValue( roSim.fGetTempC( ) ) )
      bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Save information to a Simulation object.
//
// Argument List:
//   roSim - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PnlAnaBase::bSave( Simulation & roSim )
{
  wxStringTokenizer  ostk1;
  wxString  os1;
  float     f1;
  size_t    szt1;
  bool      bRtn;

  m_osErrMsg.Empty( );

  roSim.ClrSimCmds( ); // Clear simulation commands attribute 
  roSim.ClrSrcCpnt( ); // Clear source component attribute

  // Set the simulation type
  if( ! roSim.bSetAnaType( m_eAnaType ) )
    SetErrMsg( wxT("Invalid simulation type specifier.") );

  // Set the sweep parameters
  f1 = (float) m_oPnlStart.dfGetValue( );
  if( ! roSim.bSetSwpStart( f1 ) )
    SetErrMsg( wxT("Invalid sweep start value.") );
  f1 = (float) m_oPnlStop.dfGetValue( );
  if( ! roSim.bSetSwpStop( f1 ) )
    SetErrMsg( wxT("Invalid sweep stop value.") );
  if( roSim.fGetSwpStop( ) < roSim.fGetSwpStart( ) )
    SetErrMsg( wxT("Start value greater than stop value is not permitted.") );    
  f1 = (float) m_oPnlStep.dfGetValue( );
  if( f1 > 0.0 )
  {
    if( roSim.fGetSwpStop( ) != roSim.fGetSwpStart( ) )
      if( f1 > fabs( roSim.fGetSwpStop( ) - roSim.fGetSwpStart( ) ) )
        SetErrMsg( wxT("Step size is greater than sweep range.") );
    if( ! roSim.bSetSwpStep( f1 ) )
      SetErrMsg( wxT("Invalid sweep step value.") );
  }
  else
    SetErrMsg( wxT("Step size less than or equal to zero is not permitted.") );    

  // Set the component to be used as the sweep source
  if( bIsCreatedSrc( ) )
  {
    if( m_oChoSrcCpnt.GetSelection( ) > 0 )
    { // Find the component in the net list
      for( szt1=0; szt1<roSim.roasGetCpnts( ).GetCount( ); szt1++ )
      {
        os1 = roSim.roasGetCpnts( ).Item( szt1 );
        if( os1.BeforeFirst( wxT(' ') ) == m_oChoSrcCpnt.GetStringSelection( ) )
          break;
      }
      // Create the new component definition line
      ostk1.SetString( os1 );
      os1.Empty( );
      for( szt1=0; szt1<3; szt1++ )
        os1 << ostk1.GetNextToken( ) << wxT(' ');
      if( m_oPnlSrcLvl.GetParent( ) != NULL )
           os1 << m_oPnlSrcLvl.rosGetValue( );
      else os1 << wxT('0');
    }
    else
    {
      if( m_oChoSrcCpnt.IsShown( ) && eGetAnaType( )!=Simulation::eANA_OP )
      {
        os1.Empty( );
        SetErrMsg( wxT("No source component has been selected.") );
      }
    }
    roSim.bSetSrcCpnt( os1 );
  }

  // Set the parameters to be calculated
  for( szt1=Simulation::ePAR_FST; szt1<=Simulation::ePAR_LST; szt1++ )
    roSim.bSetOutPar( (Simulation::eParType) szt1, FALSE );
  bRtn = FALSE;
  if( m_oCbxVoltage.GetValue( ) )
    { roSim.bSetOutPar( Simulation::ePAR_VLT, TRUE ); bRtn = TRUE; }
  if( m_oCbxCurrent.GetValue( ) )
    { roSim.bSetOutPar( Simulation::ePAR_CUR, TRUE ); bRtn = TRUE; }
  if( m_oCbxPower  .GetValue( ) )
    { roSim.bSetOutPar( Simulation::ePAR_PWR, TRUE ); bRtn = TRUE; }
  if( m_oCbxResist .GetValue( ) )
    { roSim.bSetOutPar( Simulation::ePAR_RES, TRUE ); bRtn = TRUE; }
  if( ! bRtn ) SetErrMsg( wxT("No parameters have been selected.") );

  // Set the complex parts of the parameters to derive
  if( eGetAnaType( ) == Simulation::eANA_AC )
  {
    for( szt1=Simulation::eCPX_FST; szt1<=Simulation::eCPX_LST; szt1++ )
      roSim.bSetOutCpx( (Simulation::eCpxType) szt1, FALSE );
    bRtn = FALSE;
    if( m_oCbxMag.GetValue( ) )
      { roSim.bSetOutCpx( Simulation::eCPX_MAG,   TRUE ); bRtn = TRUE; }
    if( m_oCbxPhase.GetValue( ) )
      { roSim.bSetOutCpx( Simulation::eCPX_PHASE, TRUE ); bRtn = TRUE; }
    if( m_oCbxReal.GetValue( ) )
      { roSim.bSetOutCpx( Simulation::eCPX_REAL,  TRUE ); bRtn = TRUE; }
    if( m_oCbxImag.GetValue( ) )
      { roSim.bSetOutCpx( Simulation::eCPX_IMAG,  TRUE ); bRtn = TRUE; }
    if( ! bRtn ) SetErrMsg( wxT("No complex parts have been selected.") );
    if( m_oCbxMagDb.GetValue( ) )
        roSim.bSetOutCpx( Simulation::eCPX_MAGDB, TRUE );
  }

  // Set the analysis temperature
  if( bIsCreatedTemp( ) && m_oSbxTemp.IsShown( ) )
  {
    f1 = (float) m_oPnlTemp.dfGetValue( );
    if( ! roSim.bSetTempC( f1 ) )
      SetErrMsg( wxT("Invalid analysis temperature value.") );
  }

  return( bIsOk( ) );
}

//*****************************************************************************
